/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.gui.stepsettings;

import edu.udo.cs.miningmart.gui.application.IteratorListener;

/**
 *An <code>IteratorListener</code>for the <code>StepLoopPanel</code>.
 * @author Daniel Hakenjos
 * @version $Id: LoopListener.java,v 1.3 2006/04/11 14:10:13 euler Exp $
 */
public class LoopListener implements IteratorListener, NrOfLoopsListener{
	
	private StepParameterTable steptable;

	private StepSettingsPanel panel;	
 
    /**
     *Constructs a new <code>LoopListener</code>.
     */
	public LoopListener(StepSettingsPanel panel,StepParameterTable table){
		this.steptable=table;
		this.panel=panel;
	}

	/**
     *Sets the loopnumber in the <code>StepParameterTable</code>.
	 * @see edu.udo.cs.miningmart.gui.application.IteratorListener#iterationChanged(String, int)
	 */
	public void iterationChanged(String data, int index) {
		steptable.setLoopNumber(Integer.parseInt(data));
	}
	
	

	/**
	 * Sets the number of loops in the <code>StepParameterTable</code>.
	 * @see edu.udo.cs.miningmart.gui.stepsettings.NrOfLoopsListener#numberOfLoopsChanged(int)
	 */
	public void numberOfLoopsChanged(int nrOfLoops) {
		steptable.setNrOfLoops(nrOfLoops);
		panel.enableCancelSave(true);
		panel.setValidity(StepSettingsPanel.NOT_TESTED);
	}

}
/*
$Log: LoopListener.java,v $
Revision 1.3  2006/04/11 14:10:13  euler
Updated license text.

Revision 1.2  2006/04/06 16:31:12  euler
Prepended license remark.

Revision 1.1  2006/01/03 09:54:15  hakenjos
Initial version!

*/
