/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.m4.utils;
import java.util.Collection;

import edu.udo.cs.miningmart.exception.M4Exception;
import edu.udo.cs.miningmart.m4.core.Columnset;
import edu.udo.cs.miningmart.m4.core.ForeignKey;
import edu.udo.cs.miningmart.m4.core.M4Object;

/**
 * Helper class for standard communication between <code>Columnset</code> (container)
 * and <code>ForeignKey</code>. Each <code>ForeignKey</code> has two links to
 * <code>Columnset</code>s, one for the primary key columnset and one for the
 * foreign key columnset. This class is used for the link to the foreign key
 * columnset.
 * 
 * @author Martin Scholz
 * @version $Id: InterM4ForeignColumnsetForeignKey.java,v 1.4 2006/04/11 14:10:10 euler Exp $
 */
public class InterM4ForeignColumnsetForeignKey extends InterM4Communicator {
	/**
	 * @param fKey a <code>ForeignKey</code>
	 * @return the current container object
	 * @see InterM4Communicator#getSingleRef(M4Object)
	 */
	public Object getSingleRef(M4Object fKey) throws M4Exception {
		return ((ForeignKey) fKey).getForeignKeyColumnset();
	}
	
	/**
	 * @param columnset a <code>Columnset</code>
	 * @return the container object's collection
	 * 
	 * @see InterM4Communicator#getCollection(M4Object)
	 */
	public Collection getCollection(Object columnset) throws M4Exception {
		return ((Columnset) columnset).getForeignKeysWhereThisIsFkCs();
	}
	
	/**
	 * @param fKkey a <code>ForeignKey</code>
	 * @param columnset a <code>Columnset</code>
	 * 
	 * @see InterM4Communicator#setSingleRefPrimitive(M4Object, M4Object)
	 */
	public void setSingleRefPrimitive(M4Object fKey, Object columnset) throws M4Exception {
		((ForeignKey) fKey).primitiveSetForeignKeyColumnset((Columnset) columnset);
	}
}
/*
 * Historie
 * --------
 * 
 * $Log: InterM4ForeignColumnsetForeignKey.java,v $
 * Revision 1.4  2006/04/11 14:10:10  euler
 * Updated license text.
 *
 * Revision 1.3  2006/04/06 16:31:10  euler
 * Prepended license remark.
 *
 * Revision 1.2  2006/03/20 16:44:21  euler
 * Bugfixes
 *
 * Revision 1.1  2006/01/03 09:54:04  hakenjos
 * Initial version!
 *
 */
