/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.operator;

import edu.udo.cs.miningmart.exception.M4CompilerError;
import edu.udo.cs.miningmart.exception.M4Exception;
import edu.udo.cs.miningmart.m4.Column;
import edu.udo.cs.miningmart.m4.Value;

/**
 * @author Timm Euler
 * @version $Id: LogScaling.java,v 1.6 2006/04/11 14:10:12 euler Exp $
 */
public class LogScaling extends Scaling  {

    public Value getLogBase() throws M4CompilerError {
    	return (Value) this.getSingleParameter("LogBase", this.getCurrentLoopNumber());
    }

    public String generateSQL(Column targetAttributeColumn) throws M4CompilerError
    {
	    // replace 0 by NULL
	    // return "to_number(decode(" + input + ", 0, NULL, log(" + logBase.value + ", " + input + ")))";
	    // Check Statistics if, min > 0:
	    try {
	    	targetAttributeColumn.updateStatistics();
		    Double min = new Double(targetAttributeColumn.getMinValue());
		    if (min.doubleValue() > 0)
		       // Produces an SQL-Exception if value is 0
	    	   return "log(" + this.getLogBase().getValue() + ", " + targetAttributeColumn.getSQLPlusLocation() + ")";
		    else
		        throw new M4CompilerError("Error: Log scaling applied to Column that contains negative or zero values!");
	    }
	    catch (M4Exception e) {
	    	throw new M4CompilerError(e.getMessage());
	    }
    };

}
/*
 * Historie
 * --------
 * 
 * $Log: LogScaling.java,v $
 * Revision 1.6  2006/04/11 14:10:12  euler
 * Updated license text.
 *
 * Revision 1.5  2006/04/06 16:31:11  euler
 * Prepended license remark.
 *
 * Revision 1.4  2006/03/30 16:07:13  scholz
 * fixed author tags for release
 *
 * Revision 1.3  2006/03/23 11:13:44  euler
 * Improved exception handling.
 *
 * Revision 1.2  2006/01/12 20:35:18  scholz
 * bugfix statistics
 *
 * Revision 1.1  2006/01/03 09:54:21  hakenjos
 * Initial version!
 *
 */

/*
 * Old Historie
 * ------------
 *
 * Revision 1.11  2003/06/05 09:52:02  euler
 * Organised imports.
 *
 * Revision 1.10  2002/10/25 13:43:27  euler
 * Changed Sql Definitions that FeatureConstruction
 * operators create. Adapted the Join operators.
 *
 * Revision 1.9  2002/10/02 17:57:47  scholz
 * beautified autoLoad().
 * getSingleParameter(..) added.
 *
 * Revision 1.8  2002/10/01 14:26:10  scholz
 * Operator.getParameter() now always outputs an M4Object[]-object.
 *
 * Revision 1.7  2002/09/05 17:06:29  scholz
 * cleaned up some more operators (wrt. autoload), e.g.
 * Scaling and sub-classes and some MissingValues operators.
 *
 * Revision 1.6  2002/08/05 10:35:47  euler
 * Restructured the operator hierarchy: introduction of SingleCSOperator
 * and MultipleCSOperator. Changed this class accordingly.
 *
 * Revision 1.5  2002/06/06 09:59:58  euler
 * Made operator loopable. Not tested yet.
 *
 * Revision 1.4  2002/06/03 12:02:37  euler
 * Restructured FeatureConstruction.
 *
 * Revision 1.3  2002/05/15 10:36:42  euler
 * First version that compiles.
 *
 * Revision 1.2  2002/05/07 13:06:56  wiese
 * get+set methods
 *
 * Revision 1.1  2002/04/30 13:00:28  wiese
 * compileable version
 *
 * Revision 1.6  2002/04/19 15:23:12  wiese
 * Initialversion nach der uebergabe
 *
 */

